==> free-sa-1.2.1.tar.gz: free-sa-1.2.1/src/work/w_fs.c <==
#include "global.h"


#if DEBUG>=3
int ofcount = 0;
#endif


#define F_SAUNLINK "SAunlink"
void SAunlink(const char *fname) {
    if (unlink(fname)) SAexit(F_SAUNLINK, "%s. %s", fname, strerror(errno));
}


#define F_SAFOPENI "SAfopenI"
FILE *SAfopenI(const char *mode, const char *fname) {
    FILE *retval;
    
#if DEBUG>=3
    ofcount++;
    SAdebug(F_SAFOPENI, "(%d) %s", ofcount, fname);
#endif
    
    if ((retval = fopen(fname, mode)) == NULL) SAexit(F_SAFOPENI, "%s. %s", fname, strerror(errno));
    
    return retval;
}


FILE *SAfopen(const char *mode, const char *fmt, ...) {
    static char ffname[MAXFILENAME];
    va_list args;
    
    va_start(args, fmt);
    vsprintf(ffname, fmt, args);
    va_end(args);

    return SAfopenI(mode, ffname);
}


FILE *SAfopenH(const char *pathtoroot, char cssprefix, const char *rname, char *sorder, char showperiod, const char *mode, const char *fmt, ...) {
    FILE *retval;
    char ffname[MAXFILENAME];
    va_list args;
    
    va_start(args, fmt);
    vsprintf(ffname, fmt, args);
    va_end(args);

    retval = SAfopenI(mode, ffname);

    fprintf(retval, "\
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n\
<html>\n\
<head>\n\
\t<title>%s</title>\n\
\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n\
\t<meta name=\"Generator\" content=\"%s %s\">\n\
\t<link rel=\"stylesheet\" href=\"%s%csa.css\" type=\"text/css\">\n\
\t<script language=\"JavaScript\" src=\"%ssa.js\" type=\"text/javascript\"></script>\n\
</head>\n\
<body>\n\
\t<center>\n\
", rname, icfg.CharSet, PROGNAME, PROGVERSION, pathtoroot, cssprefix, pathtoroot);

    if (cfg.Logo != NULL) fprintf(retval, "\t<img src=\"%s\" alt=\"\">\n", cfg.Logo);
    fputs("\t<table cellpadding=0 cellspacing=0>\n\t<tr><td class=el>\n", retval);
    if (showperiod) fprintf(retval, "\t<tr><td class=th><b>%s:</b> %s\n", loc->Period, icfg.Period);
    if (sorder != NULL) fprintf(retval, "\t<tr><td class=th><b>%s:</b> %s\n", loc->SortFieldNOrder, sorder);
    
    fprintf(retval, "\
\t<tr><th>%s\n\
\t<tr><td class=el>\n\
\t</table>\n\
\t<table cellpadding=0 cellspacing=1>\n\
", rname);

    return retval;
}


#define F_SAFCLOSE "SAfclose"
void SAfclose(FILE *ifile, const char *fname) {
#if DEBUG>=3
    ofcount--;
#endif
    if (fclose(ifile)) SAexit(F_SAFCLOSE, "%s. %s", fname, strerror(errno));
}


void SAfcloseU(FILE *ifile, const char *fname) {
    SAfclose(ifile, fname);
    SAunlink(fname);
}


void SAfcloseH(FILE *ifile, const char *fname) {
    fputs("\n\t</table>\n\t</center>\n", ifile);
    if (cfg.ShowInfo) fprintf(ifile, "<hr><i>%s <a href=\"%s\">%s %s</a></i>\n", loc->GeneratedBy, PROGURL, PROGNAME, PROGVERSION);
    fputs("</body>\n</html>\n", ifile);
    
    SAfclose(ifile, fname);
}


#define F_SAFSEEK "SAfseek"
void SAfseek(FILE *stream, SAOFF_T offset, int whence, const char *fname) {
    if (SAfseekI(stream, offset, whence)) SAexit(F_SAFSEEK, "%s. %s", fname, strerror(errno));
}


#define F_SAFTELL "SAftell"
SAOFF_T SAftell(FILE *stream) {
    SAOFF_T retval;
    if ((retval = SAftellI(stream)) == -1) SAexit(F_SAFTELL, "%s", strerror(errno));
    return retval;
}


#define F_SAFREADMC "SAfreadMC"
/* Read file to memory with allocating memory for it and saving stat to 1st arg */
void *SAfreadMC(struct stat *istat, const char *fmt, ...) {
    void *tBuf;
    FILE *tFile;
    struct stat *tStat;
    char tFileName[MAXFILENAME];
    va_list args;
    
    va_start(args, fmt);
    vsprintf(tFileName, fmt, args);
    va_end(args);
    
    if (istat == NULL) tStat = (struct stat *)SAmalloc(sizeof(struct stat));
    else tStat = istat;
    
    tFile = SAfopenI("r", tFileName);
    if (fstat(fileno(tFile), tStat)) return NULL;
    
    tBuf = SAmalloc(tStat->st_size);
    clearerr(tFile);
    fread(tBuf, tStat->st_size, 1L, tFile);
    if (ferror(tFile)) SAexit(F_SAFREADMC, tFileName);
    SAfclose(tFile, tFileName);
    
    return tBuf;
}


/* Recommended by POSIX seekdir man */
short int SAgetdirecount(DIR *iDir) {
    short int retval = 0;
    while (readdir(iDir)) retval++;
    
    return retval;
}


struct dirent *SAreaddir(DIR *iDir, short int iloc) {
    if (iloc > 0) {
	rewinddir(iDir);
	while (--iloc > 0) readdir(iDir);
	return readdir(iDir);
    } else return NULL;
}

==> free-sa-1.2.4.tar.gz: free-sa-1.2.4/src/work/w_fs.c <==
#include "global.h"


#if DEBUG>=3
int ofcount = 0;
#endif


#define F_SAUNLINK "SAunlink"
void SAunlink(const char *fname) {
    if (unlink(fname)) SAexit(F_SAUNLINK, "%s. %s", fname, strerror(errno));
}


#define F_SAFOPENI "SAfopenI"
FILE *SAfopenI(const char *mode, const char *fname) {
    FILE *retval;
    
#if DEBUG>=3
    ofcount++;
    SAdebug(F_SAFOPENI, "(%d) %s", ofcount, fname);
#endif
    
    if ((retval = fopen(fname, mode)) == NULL) SAexit(F_SAFOPENI, "%s. %s", fname, strerror(errno));
    
    return retval;
}


#define F_SAFOPEN "SAfopen"
FILE *SAfopen(const char *mode, const char *fmt, ...) {
    static char tfname[MAXFILENAME];
    va_list args;
    
    va_start(args, fmt);
    if (vsnprintf(tfname, sizeof(tfname), fmt, args) < 0) SAexit(F_SAFOPEN, "vsnprinf");
    va_end(args);

    return SAfopenI(mode, tfname);
}


#define F_SAFOPENH "SAfopenH"
FILE *SAfopenH(const char *pathtoroot, char cssprefix, const char *rname, char *sorder, char showperiod, const char *mode, const char *fmt, ...) {
    FILE *retval;
    char tfname[MAXFILENAME];
    va_list args;
    
    va_start(args, fmt);
    if (vsnprintf(tfname, sizeof(tfname), fmt, args) < 0) SAexit(F_SAFOPENH, "vsnprinf");
    va_end(args);

    retval = SAfopenI(mode, tfname);

    fprintf(retval, "\
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n\
<html>\n\
<head>\n\
\t<title>%s</title>\n\
\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n\
\t<meta name=\"Generator\" content=\"%s %s\">\n\
\t<link rel=\"stylesheet\" href=\"%s%csa.css\" type=\"text/css\">\n\
\t<script language=\"JavaScript\" src=\"%ssa.js\" type=\"text/javascript\"></script>\n\
</head>\n\
<body>\n\
\t<center>\n\
", rname, icfg.CharSet, PROGNAME, PROGVERSION, pathtoroot, cssprefix, pathtoroot);

    if (cfg.Logo != NULL) fprintf(retval, "\t<img src=\"%s\" alt=\"\">\n", cfg.Logo);
    fputs("\t<table cellpadding=0 cellspacing=0>\n\t<tr><td class=el>\n", retval);
    if (showperiod) fprintf(retval, "\t<tr><td class=th><b>%s:</b> %s\n", loc->Period, icfg.Period);
    if (sorder != NULL) fprintf(retval, "\t<tr><td class=th><b>%s:</b> %s\n", loc->SortFieldNOrder, sorder);
    
    fprintf(retval, "\
\t<tr><th>%s\n\
\t<tr><td class=el>\n\
\t</table>\n\
\t<table cellpadding=0 cellspacing=1>\n\
", rname);

    return retval;
}


#define F_SAFCLOSE "SAfclose"
void SAfclose(FILE *ifile, const char *fname) {
#if DEBUG>=3
    ofcount--;
#endif
    if (fclose(ifile)) SAexit(F_SAFCLOSE, "%s. %s", fname, strerror(errno));
}


void SAfcloseU(FILE *ifile, const char *fname) {
    SAfclose(ifile, fname);
    SAunlink(fname);
}


void SAfcloseH(FILE *ifile, const char *fname) {
    fputs("\n\t</table>\n\t</center>\n", ifile);
    if (cfg.ShowInfo) fprintf(ifile, "<hr><i>%s <a href=\"%s\">%s %s</a></i>\n", loc->GeneratedBy, PROGURL, PROGNAME, PROGVERSION);
    fputs("</body>\n</html>\n", ifile);
    
    SAfclose(ifile, fname);
}


#define F_SAFSEEK "SAfseek"
void SAfseek(FILE *stream, SAOFF_T offset, int whence, const char *fname) {
    if (SAfseekI(stream, offset, whence)) SAexit(F_SAFSEEK, "%s. %s", fname, strerror(errno));
}


#define F_SAFTELL "SAftell"
SAOFF_T SAftell(FILE *stream) {
    SAOFF_T retval;
    if ((retval = SAftellI(stream)) == -1) SAexit(F_SAFTELL, "%s", strerror(errno));
    return retval;
}


#define F_SAFREADMC "SAfreadMC"
/* Read file to memory with allocating memory for it, terminating it with '\0' and saving stat to 1st arg */
void *SAfreadMC(struct stat *istat, const char *fmt, ...) {
    void *tBuf;
    FILE *tFile;
    struct stat *tStat;
    char tfname[MAXFILENAME];
    va_list args;
    
    va_start(args, fmt);
    if (vsnprintf(tfname, sizeof(tfname), fmt, args) < 0) SAexit(F_SAFREADMC, "vsnprinf");
    va_end(args);
    
    if (istat == NULL) tStat = (struct stat *)SAmalloc(sizeof(struct stat));
    else tStat = istat;
    
    tFile = SAfopenI("r", tfname);
    /* return NULL if file have problems */
    if (fstat(fileno(tFile), tStat)) return NULL;
    
    /* next 2 lines is for functions parsing buffer as string */
    tBuf = SAmalloc(tStat->st_size + 1L);
    ((char *)tBuf)[tStat->st_size] = '\0';
    clearerr(tFile);
    fread(tBuf, tStat->st_size, 1L, tFile);
    if (ferror(tFile)) SAexit(F_SAFREADMC, tfname);
    SAfclose(tFile, tfname);
    
    return tBuf;
}


/* Recommended by POSIX seekdir man */
short int SAgetdirecount(DIR *iDir) {
    short int retval = 0;
    while (readdir(iDir)) retval++;
    return retval;
}


struct dirent *SAreaddir(DIR *iDir, short int iloc) {
    if (iloc > 0) {
	rewinddir(iDir);
	while (--iloc > 0) readdir(iDir);
	return readdir(iDir);
    } else return NULL;
}
